---------------------------------------------------
-- Structured baud rate generator for the USART peripheral
-- Count Enable (CE) and asynchronous active- high claer direct (CD)
-- 4 cascaded modules working in series to produce the following 
-- outputs from the Altera DE0 board's 50 MHz oscillator.
-- OSC_CLK_out  (divider by 1) --> 50 MHz
-- RX_CLK (divider by "n" from the input OSC_CLK_in). n = 326 to generate a (9600 b/s) x 8 clock for the receiver
-- TX_CLK (divider by 8)
-- CLK_100Hz (divider by 96 for example, to generate the 100 Hz cloxk for the debouncing filter)
-- CLK_1Hz_SQUARED 
---------------------------------------------------

LIBRARY IEEE;
   USE IEEE.STD_LOGIC_1164.ALL;
   USE IEEE.STD_LOGIC_ARITH.ALL;
   USE IEEE.STD_LOGIC_UNSIGNED.ALL;
	
ENTITY BAUD_RATE_GENERATOR IS  
      PORT(	CD,CE				: 	IN  std_logic;
			OSC_CLK_in			: 	IN  std_logic;
			OSC_CLK_out			:	OUT std_logic;
			RX_CLK				:	OUT std_logic;         
			TX_CLK				:	OUT std_logic;
			CLK_100Hz			:	OUT std_logic;
			CLK_1Hz_SQUARED		:	OUT std_logic
          );
END BAUD_RATE_GENERATOR ;

ARCHITECTURE schematic OF BAUD_RATE_GENERATOR  IS  
-- Components
	COMPONENT Freq_Div_326 IS                 
		PORT(	CD,CLK,CE	: 	IN  std_logic;
				TC326		:	OUT std_logic
		);
	END COMPONENT;	

	COMPONENT Freq_Div_8 IS                 
		PORT(	CD,CLK,CE	: 	IN  std_logic;
				TC8			:	OUT std_logic
		);
	END COMPONENT;	

	COMPONENT Freq_Div_100 IS                 
		PORT(	CD,CLK,CE	: 	IN  std_logic;
				TC100		:	OUT std_logic
		);
	END COMPONENT;	
 
	COMPONENT Freq_Div_96 IS                 
		PORT(	CD,CLK,CE	: 	IN  std_logic;
				TC96		:	OUT std_logic
		);
	END COMPONENT;	
	  		  	  
	COMPONENT T_Flip_Flop IS
      Port (	
			CLK 	: IN	STD_LOGIC;
			CD 		: IN	STD_LOGIC;
			T 		: IN	STD_LOGIC;
			Q 		: OUT	STD_LOGIC
		);
	END COMPONENT;					

-- Signals for connecting components together (just the internal wires)
	SIGNAL OSC_CLK					:	std_logic;
	SIGNAL RX_CLK_x2, TX_CLK_x2 	:	std_logic;
	SIGNAL CLK_100Hz_x2 			:	std_logic;
	SIGNAL CLK_1Hz_x2 				:	std_logic;
		
	BEGIN  
-- Instantiation of components

	CHIP1	: 	Freq_Div_326
	PORT MAP (	   	
-- from component name 	=> to signal or port name
			CLK		=> OSC_CLK,
			CD 		=> CD,
			CE 		=> CE,
			TC326	=> RX_CLK_x2					  
		);

	CHIP2	:  Freq_Div_8
		PORT MAP (
-- from component name 	=> to signal or port name
			CLK		=> OSC_CLK,
			CD 		=> CD,
			CE 		=> RX_CLK_x2,
			TC8 	=> TX_CLK_x2					  
		);

	CHIP3	: 	Freq_Div_100
		PORT MAP (
-- from component name 	=> to signal or port name
			CLK		=> OSC_CLK,
			CD 		=> CD,
			CE 		=> TX_CLK_x2,
			TC100 	=> CLK_100Hz_x2					  
		);

	CHIP4	:	Freq_Div_96
		PORT MAP (
-- from component name 	=> to signal or port name
			CLK		=> OSC_CLK,
			CD 		=> CD,
			CE 		=> CLK_100Hz_x2,
			TC96 	=> CLK_1Hz_x2					  
		);

	Chip5	: T_Flip_Flop
		PORT MAP (
	-- from component name 	=> to signal or port name	
			CLK		=> OSC_CLK,
			CD 		=> CD,
			T		=> RX_CLK_x2,
			Q 		=> RX_CLK				  
		);		

	Chip6	: T_Flip_Flop
		PORT MAP (
	-- from component name 	=> to signal or port name	
			CLK		=> OSC_CLK,
			CD 		=> CD,
			T		=> TX_CLK_x2,
			Q 		=> TX_CLK				  
		);	
		
	Chip7	: T_Flip_Flop
		PORT MAP (
	-- from component name 	=> to signal or port name	
			CLK		=> OSC_CLK,
			CD 		=> CD,
			T		=> CLK_100Hz_x2,
			Q 		=> CLK_100Hz				  
		);	
		
	Chip8	: T_Flip_Flop
		PORT MAP (
	-- from component name 	=> to signal or port name	
			CLK		=> OSC_CLK,
			CD 		=> CD,
			T		=> CLK_1Hz_x2,
			Q 		=> CLK_1Hz_SQUARED				  
		);	
			
-- connections and logic between components
	
	-- The circuit's signals that have to be connected to input ports 
	OSC_CLK <= OSC_CLK_in;
	
	-- The output ports that have to be connected to internal signals 
	OSC_CLK_out	<= OSC_CLK;
	
END schematic ;
