-- The complete UART module
-- 
-- NOTES: 	The debouncing filter is active-high (PB)
-- 			The Baud Rate Generator has an active-high clear direct (CD)


LIBRARY ieee;
USE  IEEE.STD_LOGIC_1164.all;
USE  IEEE.STD_LOGIC_UNSIGNED.all;

ENTITY UART_module IS
	PORT (	
		OSC_CLK_in		: 	IN  STD_LOGIC;	
		CD				:	IN  STD_LOGIC;
		Enable_TX		:	IN  STD_LOGIC;
		TX_IN			: 	IN 	STD_LOGIC_VECTOR (7 DOWNTO 0);		
		ST				: 	IN 	STD_LOGIC;
		Serial_out		:	OUT  STD_LOGIC;
		ET				:  	OUT	STD_LOGIC;

		Enable_RX		:	IN  STD_LOGIC;
	   	Serial_in   	:	IN  STD_LOGIC;
		RX_OUT			:	OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
		Error			:	OUT STD_LOGIC;
		New_Data		:	OUT STD_LOGIC;
		CLK_1Hz_SQUARED	:	OUT STD_LOGIC -- The 1 Hz LED to show that the system works
	);
END UART_module;   
		

ARCHITECTURE structural OF UART_module IS
-- The Four components
COMPONENT BAUD_RATE_GENERATOR IS
	PORT(	
		CD,CE			: 	IN  std_logic;
		OSC_CLK_in		: 	IN  std_logic;
		OSC_CLK_out		:	OUT std_logic;
		RX_CLK			:	OUT std_logic;         
		TX_CLK			:	OUT std_logic;
		CLK_100Hz		:	OUT std_logic;
		CLK_1Hz_SQUARED	:	OUT std_logic
	);
END COMPONENT;      

COMPONENT deboucing_filter IS
	Port (	
		CLK 	: IN	STD_LOGIC;
		PB_L 	: IN	STD_LOGIC;
		CD   	: IN STD_LOGIC;
		QA 		: OUT	STD_LOGIC;
		QB		: OUT	STD_LOGIC
	);
END COMPONENT;      

COMPONENT Transmitter_unit IS
	PORT (
		CLK 		: 	IN 	STD_LOGIC;
		CD			: 	IN 	STD_LOGIC;
		TX_IN		: 	IN 	STD_LOGIC_VECTOR (7 DOWNTO 0);		
		ST			: 	IN 	STD_LOGIC;
		Enable_TX 	: 	IN 	STD_LOGIC;
		ET			:  	OUT	STD_LOGIC;
		Serial_out	: 	OUT STD_LOGIC
	);
END COMPONENT;      

COMPONENT Receiver_unit IS
	PORT (	
		CLK			: 	IN  STD_LOGIC;	
		CD			:	IN  STD_LOGIC;
		Enable_RX	:	IN  STD_LOGIC;
	   	Serial_in   :	IN  STD_LOGIC;
		RX_OUT		:	OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
		Error		:	OUT STD_LOGIC;
		New_Data	:	OUT STD_LOGIC
	);
END COMPONENT;   


-- The signals to connect the modules:
	SIGNAL TX_CLK 		: 	STD_LOGIC;
	SIGNAL RX_CLK		: 	STD_LOGIC; 
	SIGNAL CLK_100Hz	: 	STD_LOGIC;
	SIGNAL OSC_CLK_out	: 	STD_LOGIC;   
	SIGNAL ST_Pulse		:	STD_LOGIC;    -- Only one clock pulse (10 ms - 100 Hz )
	SIGNAL ST_Level		:	STD_LOGIC; 	  -- At '1' as long as the user is clikling      
	SIGNAL ST_L			:	STD_LOGIC;	-- The debouncing filter was designed with an active low input 
	
	
BEGIN
-- Instantiation of the components:
Chip1 : BAUD_RATE_GENERATOR
	PORT MAP (
-- from component name 	=> to signal or port name
		CD				=>	CD,
		CE				=>	'1',
		OSC_CLK_in		=>	OSC_CLK_in,
		OSC_CLK_out		=>	OSC_CLK_out,
		RX_CLK			=>	RX_CLK,
		TX_CLK			=>	TX_CLK,
		CLK_100Hz		=>	CLK_100Hz,
		CLK_1Hz_SQUARED	=>	CLK_1Hz_SQUARED			  
	);

Chip2 : deboucing_filter
	PORT MAP (
-- from component name 	=> to signal or port name 
		CLK				=> 	CLK_100Hz,	
		PB_L   			=>	ST_L,	
		CD   			=>	CD,
		QA 				=>	ST_Pulse,
		QB				=>	ST_Level	
	);

Chip3 : Transmitter_unit
-- from component name 	=> to signal or port name 
	PORT MAP(
		CLK 			=> 		TX_CLK,	
		CD				=> 		CD,			
		TX_IN			=> 		TX_IN,					
		ST				=> 		ST_Pulse,				
		Enable_TX 		=> 		Enable_TX, 
		ET				=> 		ET,		
		Serial_out		=> 		Serial_out		
	);

Chip4 : Receiver_unit
-- from component name 	=> to signal or port name 
	PORT MAP(
		CLK 			=> 		RX_CLK,	
		CD				=> 		CD,		
		Enable_RX  		=> 		Enable_RX,
	   	Serial_in		=> 		Serial_in,
		RX_OUT			=> 		RX_OUT,	
		Error			=> 		Error,	
		New_Data		=>		New_Data
	);


-- Extra logic 

ST_L <= NOT (ST);

END structural ;